@@ -2,9 +2,9 @@
 Module      : HsLua.Marshalling
 Copyright   : © 2007–2012 Gracjan Polak;
               © 2012–2016 Ömer Sinan Ağacan;
-              © 2017-2022 Albert Krewinkel
+              © 2017-2023 Albert Krewinkel
 License     : MIT
-Maintainer  : Albert Krewinkel <tarleb+hslua@zeitkraut.de>
+Maintainer  : Albert Krewinkel <tarleb@hslua.org>
 
 Functions to push and retrieve data to and from Lua.
 -}
@@ -42,6 +42,9 @@ module HsLua.Marshalling
   , peekFieldRaw
   , peekPair
   , peekTriple
+  , peekNilOr
+  , peekNoneOr
+  , peekNoneOrNilOr
     -- ** Lua peek monad
   , Peek (..)
   , forcePeek
