@@ -2,9 +2,9 @@
 {-# LANGUAGE TypeApplications #-}
 {-|
 Module      : HsLua.Marshalling.PeekersTests
-Copyright   : © 2020-2022 Albert Krewinkel
+Copyright   : © 2020-2023 Albert Krewinkel
 License     : MIT
-Maintainer  : Albert Krewinkel <tarleb+hslua@zeitkraut.de>
+Maintainer  : Albert Krewinkel <tarleb@hslua.org>
 
 Tests for Haskell-value retriever functions.
 -}
@@ -25,6 +25,7 @@ import Test.Tasty.QuickCheck (testProperty)
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as Char8
+import qualified Data.List.NonEmpty as NonEmpty
 import qualified Data.Map as Map
 import qualified Data.Set as Set
 import qualified Data.Text as T
@@ -278,6 +279,18 @@ tests = testGroup "Peekers"
           runPeek $ peekList peekByteString Lua.top
       ]
 
+    , testGroup "peekNonEmpty"
+      [ "empty list" =:
+        Failure "empty list" ["retrieving NonEmpty"] `shouldBeResultOf` do
+          Lua.newtable
+          runPeek $ peekNonEmpty peekBool Lua.top
+
+      , "non-empty list" =:
+        Success (5 NonEmpty.:| [23]) `shouldBeResultOf` do
+          pushLuaExpr "{ 5, 23 }"
+          runPeek $ peekNonEmpty (peekIntegral @Int) Lua.top
+      ]
+
     , testGroup "peekSet"
       [ "empty set" =:
         Success Set.empty `shouldBeResultOf` do
@@ -343,6 +356,7 @@ tests = testGroup "Peekers"
       ]
     ]
 
+
   , testGroup "combinators"
     [ testGroup "peekFieldRaw"
       [ "access field" =:
@@ -409,6 +423,48 @@ tests = testGroup "Peekers"
         Failure "all choices failed" [] `shouldBeResultOf` do
           runPeeker (choice [const $ failPeek @() "nope"]) Lua.top
       ]
+
+    , testGroup "peekNilOr"
+      [ "returns the parser result if the value is not nil" =:
+        Success (Just "a") `shouldBeResultOf` runPeek
+          (liftLua (Lua.pushstring "a") *> peekNilOr peekString Lua.top)
+      , "returns nothing if the value is nil" =:
+        Success Nothing `shouldBeResultOf` runPeek
+          (liftLua Lua.pushnil *> peekNilOr peekString Lua.top)
+      , "fails if the value is none" =:
+        Failure "string expected, got no value" [] `shouldBeResultOf` runPeek
+          (liftLua Lua.gettop >>= peekNilOr @Maybe peekString . (+1))
+      ]
+
+    , testGroup "peekNoneOr"
+      [ "returns the parser result if a value is present" =:
+        Success (Just "a") `shouldBeResultOf` runPeek
+          (liftLua (Lua.pushstring "a") *> peekNoneOr peekString Lua.top)
+      , "returns the parser result if the value is nil" =:
+        Success (Just ()) `shouldBeResultOf` runPeek
+          (liftLua Lua.pushnil *> peekNoneOr peekNil Lua.top)
+      , "returns `empty` if the value is missing" =:
+        Success Nothing `shouldBeResultOf` runPeek
+          (liftLua Lua.gettop >>= peekNoneOr @Maybe peekString . (+1))
+      , "fails if the parser cannot parse the value" =:
+        Failure "string expected, got nil" [] `shouldBeResultOf` runPeek
+          (liftLua Lua.pushnil *> peekNoneOr @Maybe peekString Lua.top)
+      ]
+    , testGroup "peekNoneOrNilOr"
+      [ "returns the parser result if a value is present" =:
+        Success (Just "a") `shouldBeResultOf` runPeek
+          (liftLua (Lua.pushstring "a") *> peekNoneOrNilOr peekString Lua.top)
+      , "returns `empty` if the value is nil" =:
+        Success Nothing `shouldBeResultOf` runPeek
+          (liftLua Lua.pushnil *> peekNoneOrNilOr @Maybe peekString Lua.top)
+      , "returns `empty` if the value is missing" =:
+        Success Nothing `shouldBeResultOf` runPeek
+          (liftLua Lua.gettop >>= peekNoneOrNilOr @Maybe peekString . (+1))
+      , "fails if the parser cannot parse the value" =:
+        Failure "string expected, got boolean" [] `shouldBeResultOf` runPeek
+          (liftLua (Lua.pushboolean True) *>
+           peekNoneOrNilOr @Maybe peekString Lua.top)
+      ]
     ]
 
   , testGroup "helper"
